import { Character, Zone } from '@prisma/client'
import ZoneRepository from '../repositories/ZoneRepository'
import ZoneService from '../services/ZoneService'
import zoneRepository from '../repositories/ZoneRepository'

type TLoadedZone = {
  zone: Zone
  characters: Character[]
}

class ZoneManager {
  private loadedZones: TLoadedZone[] = []

  // Method to initialize zone manager
  public async boot() {
    if (!(await ZoneRepository.getById(1))) {
      const zoneService = new ZoneService()
      await zoneService.createDemoZone()
    }

    const zones = await ZoneRepository.getAll()

    for (const zone of zones) {
      this.loadZone(zone)
    }

    console.log('[✅] Zone manager loaded')
  }

  // Method to handle individual zone loading
  public loadZone(zone: Zone) {
    this.loadedZones.push({
      zone,
      characters: []
    })
    console.log(`[✅] Zone ID ${zone.id} loaded`)
  }

  // Method to handle individual zone unloading
  public unloadZone(zoneId: number) {
    this.loadedZones = this.loadedZones.filter((loadedZone) => {
      return loadedZone.zone.id !== zoneId
    })
    console.log(`[❌] Zone ID ${zoneId} unloaded`)
  }

  // Getter for loaded zones
  public getLoadedZones(): TLoadedZone[] {
    return this.loadedZones
  }

  public addCharacterToZone(zoneId: number, character: Character) {
    const loadedZone = this.loadedZones.find((loadedZone) => {
      return loadedZone.zone.id === zoneId
    })
    if (loadedZone) {
      loadedZone.characters.push(character)
    }
  }

  public removeCharacterFromZone(zoneId: number, character: Character) {
    const loadedZone = this.loadedZones.find((loadedZone) => {
      return loadedZone.zone.id === zoneId
    })
    if (loadedZone) {
      loadedZone.characters = loadedZone.characters.filter((loadedCharacter) => {
        return loadedCharacter.id !== character.id
      })
    }
  }

  public updateCharacterInZone(zoneId: number, character: Character) {
    const loadedZone = this.loadedZones.find((loadedZone) => {
      return loadedZone.zone.id === zoneId
    })
    if (loadedZone) {
      const characterIndex = loadedZone.characters.findIndex((loadedCharacter) => {
        return loadedCharacter.id === character.id
      })
      if (characterIndex !== -1) {
        loadedZone.characters[characterIndex] = character
      }
    }
  }

  public getCharactersInZone(zoneId: number): Character[] {
    const loadedZone = this.loadedZones.find((loadedZone) => {
      return loadedZone.zone.id === zoneId
    })
    return loadedZone ? loadedZone.characters : []
  }

  public async getGrid(zoneId: number): Promise<number[][]> {
    const zone = this.loadedZones.find((z) => z.zone.id === zoneId)
    let grid: number[][] = []
    if (zone) {
      const eventTiles = await zoneRepository.getEventTiles(zoneId)
      const objects = await zoneRepository.getObjects(zoneId)

      // Create a grid based on the event tiles
      for (let i = 0; i < zone.zone.width; i++) {
        grid.push(Array(zone.zone.height).fill(0))
      }

      // Set the grid values based on the event tiles, these are strings
      eventTiles.forEach((eventTile) => {
        if (eventTile.type === 'BLOCK') {
          grid[eventTile.position_x][eventTile.position_y] = 0
        } else if (eventTile.type === 'WARP') {
          grid[eventTile.position_x][eventTile.position_y] = 1
        } else if (eventTile.type === 'NPC') {
          grid[eventTile.position_x][eventTile.position_y] = 2
        } else if (eventTile.type === 'ITEM') {
          grid[eventTile.position_x][eventTile.position_y] = 3
        }
      });

      objects.forEach((object) => {
        if (object.objectId === 'blank_tile') {
          grid[object.position_x][object.position_y] = 0
        } else {
          grid[object.position_x][object.position_y] = 1
        }
      });

      return grid
    }
    return []
  }
}

export default new ZoneManager()
